import gexf;


EXPORT CreateGraph := MODULE

  EXPORT graph_node_rec := record
     unsigned id;
     string label;
     unsigned size := 1;
     string color;
  END;

  EXPORT graph_edge_rec := record
	   unsigned edge_id := 0;
     unsigned source;
     unsigned target;
  END;

	EXPORT attvalue_rec := record
		string id {xpath('@id')};
		string value {xpath('@value')};
	end;

	EXPORT attvalues_rec := record
		DATASET(attvalue_rec) attribute {xpath('/attvalue')};
	end;

	EXPORT vizcolor_rec := record
		string b {xpath('@b')};
		string g {xpath('@g')};
		string r {xpath('@r')};
	end;

	EXPORT vizposition_rec := record
		string x {xpath('@x')};
		string y {xpath('@y')};
		string z {xpath('@z')} := '0';
	end;

	EXPORT vizsize_rec := record
		string value {xpath('@value')};
	end;

	EXPORT node_rec := record
		string id {xpath('@id')};
		string label {xpath('@label')};
		vizcolor_rec vizcolor {xpath('viz:color')};
		vizposition_rec vizposition {xpath('viz:position')};
		vizsize_rec vizsize {xpath('viz:size')};
		attvalues_rec attvalues;
	end;

	EXPORT nodes_rec := record
		DATASET(node_rec) node {xpath('node')};
	end;

	EXPORT node_rec tNodes(graph_node_rec l) := transform
		self.id := (string)l.id;
		self.label := l.label;
		self.vizsize.value := (string)l.size;
		self.vizcolor.b := gexf.GraphColor.GetColorB(l.color);
		self.vizcolor.g := gexf.GraphColor.GetColorG(l.color);
		self.vizcolor.r := gexf.GraphColor.GetColorR(l.color);
		self.vizposition.x := (string)random();
		self.vizposition.y := (string)random();
		self.attvalues.attribute := dataset([{'0', 'n2'}, {'1', l.label}, {'2', '1'}], attvalue_rec); 
	end;

	EXPORT edge_rec := record
	 string id {xpath('@id')};
	 string source {xpath('@source')};
	 string target {xpath('@target')};
	end;

	EXPORT edge_rec tEdges(graph_edge_rec l, integer c) := transform
		self.id := if (l.edge_id != 0, (string)l.edge_id, (string)c);
		self.source := (string)l.source;
		self.target := (string)l.target;	
	end;

	EXPORT edges_rec := record
		DATASET(edge_rec) node {xpath('edge')};
	end;

	edges_ds := dataset([{'0', '0', '1'}, {'1', '1', '2'}], edge_rec);

	EXPORT attibute_rec := record
	 string id {xpath('@id')};
	 string title {xpath('@title')};
	 string type_attr {xpath('@type')};
	end;

	EXPORT attributes_rec := record
		string class {xpath('@class')};
		string attrib_type {xpath('@type')};
		DATASET(attibute_rec) attribute {xpath('/attribute')};
	end;

	EXPORT graph_rec := record
	 string type_att {xpath('@type')};
	 //string mode {xpath('@mode')};
	 //string defaultedgetype {xpath('@defaultedgetype')};
	 attributes_rec attributes;
	 nodes_rec nodes {xpath('nodes')};
	 edges_rec edges {xpath('edges')};
	end;

	EXPORT gexf_rec := record 
	 string xmlns {xpath('@xmlns')};
	 string xmlns_viz {xpath('@xmlns:viz')};
	 graph_rec mygraph {xpath('graph')};
	end;

	EXPORT wrapper_rec := record
	 gexf_rec gexf;
	end;

				
	EXPORT getGraphgexf(DATASET(graph_node_rec) inNodes, DATASET(graph_edge_rec) inEdges) := FUNCTION

	nodes_ds := project(inNodes, tNodes(LEFT));
	edges_ds := project(inEdges, tEdges(LEFT, COUNTER));

		myds := dataset([ {
										 { 'http://www.gephi.org/gexf', 'http://www.gephi.org/gexf/viz', 
											 { 'static', { 'node', 'static', 
													[{'0', 'nodedef', 'string'},{'1', 'label', 'string'}, {'2', 'occurrences', 'integer'}],
											 }, {nodes_ds}, {edges_ds} }
										 }
										}
									], wrapper_rec);

		RETURN myds;
	END;

END;